@@ -16,6 +16,8 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
   List colWidths(n_sheets);
   List rowHeights(n_sheets);
   List wbstyleObjects(0);
+  List outlineLevels(n_sheets);
+  List colOutlineLevels(n_sheets);
 
   // loop over each worksheet file
   for(int i = 0; i < n_sheets; i++){
@@ -24,11 +26,15 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       
       colWidths[i] = List(0);
       rowHeights[i] = List(0);
+      outlineLevels[i] = List(0);
+      colOutlineLevels[i] = List(0);
       
     }else{
       
       colWidths[i] = List(0);
       rowHeights[i] = List(0);
+      outlineLevels[i] = List(0);
+      colOutlineLevels[i] = List(0);
       Reference this_worksheet(worksheets[i]);
       Reference sheet_data(this_worksheet.field("sheet_data"));
       
@@ -97,17 +103,23 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       if(cols.size() > 0){
         
         NumericVector widths;
-        IntegerVector columns;
-        CharacterVector column_hidden;  
+        IntegerVector columns_with_widths;
+        IntegerVector columns_with_groups;
+        CharacterVector column_hidden;
+        CharacterVector col_outline;
+        CharacterVector col_hidden;
         
-        for(size_t ci = 0; ci < cols.size(); ci++){
+        for (size_t ci = 0; ci < cols.size(); ci++) {
           
           double tmp_width = 0;
           std::string tmp_hidden;
           int min_c = 0;
           int max_c = 0;
+          std::string tmp_coloutline;
+
           buf = cols[ci];
-          if(buf.find("customWidth", 0) != string::npos){
+          // If either custom widths or groupings, get column index
+          if ((buf.find("customWidth", 0) != string::npos) | (buf.find("outlineLevel", 0) != string::npos)) {
             
             tmp_pos = buf.find("min=\"", 0);
             endPos = buf.find(tagEnd, tmp_pos + 5);
@@ -116,43 +128,79 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
             tmp_pos = buf.find("max=\"", 0);
             endPos = buf.find(tagEnd, tmp_pos + 5);
             max_c = atoi(buf.substr(tmp_pos + 5, endPos - tmp_pos - 5).c_str());
-            
-            tmp_pos = buf.find("width=\"", 0);
-            endPos = buf.find(tagEnd, tmp_pos + 7);
-            tmp_width = atof(buf.substr(tmp_pos + 7, endPos - tmp_pos - 7).c_str()) - 0.71;
-            
+
             tmp_pos = buf.find("hidden=\"", 0);
-            if(tmp_pos != string::npos){
+
+            if (tmp_pos != string::npos) {
               endPos = buf.find(tagEnd, tmp_pos + 8);
               tmp_hidden = buf.substr(tmp_pos + 8, endPos - tmp_pos - 8);
-            }else{
+            } else {
               tmp_hidden = "0";
             }
-            
 
-            while(min_c <= max_c){
-              widths.push_back(tmp_width);
-              columns.push_back(min_c);
-              column_hidden.push_back(tmp_hidden);
-              min_c++;
+            // If column has both a custom width and is part of a group
+            if ((buf.find("customWidth", 0) != string::npos) & (buf.find("outlineLevel", 0) != string::npos)) {
+              tmp_pos = buf.find("width=\"", 0);
+              endPos = buf.find(tagEnd, tmp_pos + 7);
+              tmp_width = atof(buf.substr(tmp_pos + 7, endPos - tmp_pos - 7).c_str()) - 0.71;
+              
+              tmp_pos = buf.find("outlineLevel=\"", 0);
+              endPos = buf.find(tagEnd, tmp_pos + 14);
+              tmp_coloutline = buf.substr(tmp_pos + 14, endPos - tmp_pos - 14);  
+
+              while (min_c <= max_c) {
+                widths.push_back(tmp_width);
+                columns_with_widths.push_back(min_c);
+                columns_with_groups.push_back(min_c);
+                column_hidden.push_back(tmp_hidden);
+                col_hidden.push_back(tmp_hidden);
+                col_outline.push_back(tmp_coloutline);
+                min_c++;
+              }
+              
+            } else if (buf.find("customWidth", 0) != string::npos) {      // Column only has a custom width
+
+              tmp_pos = buf.find("width=\"", 0);
+              endPos = buf.find(tagEnd, tmp_pos + 7);
+              tmp_width = atof(buf.substr(tmp_pos + 7, endPos - tmp_pos - 7).c_str()) - 0.71;
+
+              while (min_c <= max_c) {
+                widths.push_back(tmp_width);
+                columns_with_widths.push_back(min_c);
+                column_hidden.push_back(tmp_hidden);
+                min_c++;
+              }
+            } else {      // Column is only part of a group
+
+              tmp_pos = buf.find("outlineLevel=\"", 0);
+              endPos = buf.find(tagEnd, tmp_pos + 14);
+              tmp_coloutline = buf.substr(tmp_pos + 14, endPos - tmp_pos - 14);  
+
+              while (min_c <= max_c) {
+                columns_with_groups.push_back(min_c);
+                col_hidden.push_back(tmp_hidden);
+                col_outline.push_back(tmp_coloutline);
+                min_c++;
+              }
             }
-
           }
-          
         }
         
-        if(widths.size() > 0){
+        if (widths.size() > 0) {
           CharacterVector tmp_widths(widths);
-          tmp_widths.attr("names") = columns;
+          tmp_widths.attr("names") = columns_with_widths;
           tmp_widths.attr("hidden") = column_hidden;
           colWidths[i] = tmp_widths;
         }
-        
+
+        if (col_outline.size() > 0) {
+          CharacterVector columns_outline(col_outline);
+          columns_outline.attr("names") = columns_with_groups;
+          columns_outline.attr("hidden") = col_hidden;
+          colOutlineLevels[i] = columns_outline;
+        }
       }
       
-      
-      
-      
       /* --- Everything after sheetData --- */
       size_t pos_post = 0;
       if(has_data){
@@ -524,6 +572,8 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
         
         CharacterVector rowNumbers(row_ocs);
         CharacterVector heights(row_ocs);
+        CharacterVector outlines(row_ocs);
+        CharacterVector outline_hidden(row_ocs);
         
         
         // PULL OUT CELL AND ATTRIBUTES
@@ -531,6 +581,7 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
         pos = xml.find("<row ", 0);
         std::string htTag = " ht=\"";
         std::string attrEnd = "\"";
+        std::string posVal;
         
         while(j < row_ocs){
           
@@ -553,9 +604,40 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
             }else{  
               endPos = cell.find(attrEnd, pos + 5);
               heights[j] = cell.substr(pos + 5, endPos - pos - 5);
-            }
-            
+            };
+
+            // Is row grouped?
+            pos = cell.find("outlineLevel=\"", 0);
+            if(pos == std::string::npos){
+              outlines[j] = NA_STRING;
+            } else {
+              endPos = cell.find(attrEnd, pos + 14);
+              posVal = cell.substr(pos + 14, endPos - pos - 14);
+              // outlines[j] = posVal;
+              // Ignore if value is 0
+              if (posVal != "0") {
+                outlines[j] = posVal;
+              } else {
+                outlines[j] = NA_STRING;
+              }
+            };
             
+            // Is grouped row hidden?
+            pos = cell.find("hidden=\"", 0);
+            if(pos == std::string::npos){
+              // outline_hidden[j] = "0";   // NA_STRING;
+              outline_hidden[j] = NA_STRING;
+            } else {
+              endPos = cell.find(attrEnd, pos + 8);
+              posVal = cell.substr(pos + 8, endPos - pos - 8);
+              outline_hidden[j] = posVal;
+              // Ignore if there's no grouping for that row
+              if (!CharacterVector::is_na(outlines[j])) {
+                outline_hidden[j] = posVal;
+              } else {
+                outline_hidden[j] = NA_STRING;
+              }
+            };
             
             j++; // INCREMENT OVER OCCURENCES
             pos = nextPos;
@@ -564,12 +646,26 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
         }  // END OF CELL AND ATTRIBUTION GATHERING
         
         
-        rowNumbers = rowNumbers[!is_na(heights)];
-        if(rowNumbers.size() > 0){
-          heights = heights[!is_na(heights)];
-          heights.attr("names") = rowNumbers;
-          rowHeights[i] = heights;
+        CharacterVector heightsRows(rowNumbers);
+        CharacterVector outlineRows(rowNumbers);
+
+        heightsRows = heightsRows[!is_na(heights)];
+        if (heightsRows.size() > 0) {
+        	heights = heights[!is_na(heights)];
+        	heights.attr("names") = heightsRows;
+        	rowHeights[i] = heights;
         }
+
+        outlineRows = outlineRows[!is_na(outlines)];
+        if (outlineRows.size() > 0) {
+        	outline_hidden = outline_hidden[!is_na(outline_hidden)];
+        	outlines = outlines[!is_na(outlines)];
+        	outlines.attr("names") = outlineRows;
+        	outlines.attr("hidden") = outline_hidden;
+        	outlineLevels[i] = outlines;
+        }
+
+
         
         // styleObjects
         std::string this_sheetname = as<std::string>(sheetNames[i]);
@@ -634,6 +730,8 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
   wb.field("rowHeights") = rowHeights;
   wb.field("colWidths") = colWidths;
   wb.field("styleObjects") = wbstyleObjects;
+  wb.field("outlineLevels") = outlineLevels;
+  wb.field("colOutlineLevels") = colOutlineLevels;
 
   
   return wrap(wb);
