@@ -572,7 +572,7 @@ SEXP read_workbook(IntegerVector cols_in,
     
   }else{ // else col_names is FALSE
     char name[6];
-    for(unsigned short i =0; i < nCols; i++){
+    for(int i =0; i < nCols; i++){
       sprintf(&(name[0]), "X%hu", i+1);
        // snprintf(&(name[0]), sizeof(&(name[0])), "X%d", i+1);
       // sprintf(&(name[0]), "X%u", i+1);
